from PolyominoTASAssembly import *
from PolyominoTASTileType import *
from Cython.Compiler.Naming import self_cname


class PolyominoSystem:
    def __init__(self):
        self.Clear()
        self.binding_threshold = 2
        
    def Clear(self):
        self.binding_threshold = None
        self.PolyominoType_dict = dict()
        self.seed = PolyominoAssembly()
        self.assembly = PolyominoAssembly()

    def AddPolyominoType(self, pct):
        if pct.name in self.PolyominoType_dict:
            print(f"PolyominoType with name {pct.name} already exists in set")
        else:
            self.PolyominoType_dict[pct.name] = pct

    def ReadFromFile(self, file_name):
        self.Clear()
        xml_tree = ET.parse(file_name)
        xml_root = xml_tree.getroot()
        if not xml_root.tag == 'PolyominoSystem':
            print("Invalid XML PolyominoSystem file")
            return False
        else:
            for child in xml_root:
                if child.tag == 'bindingThreshold':
                    self.binding_threshold = int(child.text)
                elif child.tag == 'PolyominoTypes':
                    for elt_type in child:
                        pct = PolyominoType(name=None, xml_node=elt_type)
                        self.PolyominoType_dict[pct.name] = pct
                elif child.tag == 'seed':
                    for child2 in child:
                        if child2.tag == 'Polyominoes':
                            for elt_polyomino in child2:
                                p = Polyomino(polyominoType=None, xml_node=elt_polyomino, polyominoTypes=self.PolyominoType_dict)
                                self.seed.AddPolyomino(p)
                elif child.tag == 'assembly':
                    for child2 in child:
                        if child2.tag == 'Polyominoes':
                            for elt_polyomino in child2:
                                p = Polyomino(polyominoType=None, xml_node=elt_polyomino, polyominoTypes=self.PolyominoType_dict)
                                self.assembly.AddPolyomino(p)
        
        if (len(self.assembly.locationMap) == 0) and (len(self.seed.locationMap) > 0):
            self.assembly = self.seed

        self.assembly.polyominoSystem = self
        self.seed.polyominoSystem = self

        # Validation steps:
        # - if given types and seed, verify that each entry in the seed is of one of the types
        # - if given types and assembly, verify that each entry in the assembly is of one of the types
        for loc in self.seed.locationMap:
            polyomino = self.seed.locationMap[loc]
            if not polyomino.polyominoType.name in self.PolyominoType_dict:
                print(f"ERROR: seed contains Polyomino of type {polyomino.polyominoType.name} which is not in the PolyominoType set {self.PolyominoType_dict.keys()}")
                return False
        for loc in self.assembly.locationMap:
            polyomino = self.assembly.locationMap[loc]
            if not polyomino.polyominoType.name in self.PolyominoType_dict:
                print(f"ERROR: assembly contains Polyomino of type {polyomino.polyominoType.name} which is not in the PolyominoType set {self.PolyominoType_dict.keys()}")
                return False
        return True   
                        
    def WriteToFile(self, file_name):
        xml_root = ET.Element('PolyominoSystem')
        if not self.binding_threshold is None:
            elt_bt = ET.SubElement(xml_root, 'bindingThreshold')
            elt_bt.text = str(self.binding_threshold)
        if not self.PolyominoType_dict is None:
            elt_pct_set = ET.SubElement(xml_root, 'PolyominoTypes')
            for pct in self.PolyominoType_dict:
                self.PolyominoType_dict[pct].toXML(elt_pct_set)
        if not self.seed is None:
            elt_seed = ET.SubElement(xml_root, 'seed')
            elt_polyominoes = ET.SubElement(elt_seed, 'Polyominoes')
            polyominoes = self.seed.GetPolyominoes()
            for polyomino in polyominoes:
                polyomino.toXML(elt_polyominoes)
        if not self.assembly is None:
            elt_assembly = ET.SubElement(xml_root, 'assembly')
            elt_polyominoes = ET.SubElement(elt_assembly, 'Polyominoes')
            polyominoes = self.assembly.GetPolyominoes()
            for polyomino in polyominoes:
                polyomino.toXML(elt_polyominoes)

        xml_string = ET.tostring(xml_root)
        xml_dom = xml.dom.minidom.parseString(xml_string)
        
        with open(file_name, "w") as f: 
            f.write(xml_dom.toprettyxml())
        return True
